package SQ.Blocks.Indicators.CMMA;

import com.strategyquant.lib.*;
import com.strategyquant.datalib.*;
import com.strategyquant.tradinglib.*;

import SQ.Internal.IndicatorBlock;

/**
 * Indicator name as it will be displayed in UI, and its return type.
 * Possible return types:
 * ReturnTypes.Price - indicator is drawn on the price chart, like SMA, Bollinger Bands etc.
 * ReturnTypes.Number - indicator is drawn on separate chart, like CCI, RSI, MACD
 * ReturnTypes.PriceRange - indicator is price range, like ATR.
 */
@BuildingBlock(name="(CMMA) CMMA", display="CMMA (@Chart@#Lookback#,#ATRLength#)[#Shift#]", returnType = ReturnTypes.Number)
@Help("Close minus moving mverage, normalised")
@Indicator(oscillator=true, middleValue=0, min=-1000, max=1000, step=0.01)
public class CMMA extends IndicatorBlock {

	@Parameter(defaultChartIndex=0)
	public ChartData Chart;

	@Parameter(defaultValue="10", isPeriod=true, minValue=2, maxValue=1000, step=1)
	public int Lookback;

	@Parameter(defaultValue="100", isPeriod=true, minValue=2, maxValue=1000, step=1)
	public int ATRLength;

	@Output
	public DataSeries Value;

	//------------------------------------------------------------------------
	//------------------------------------------------------------------------
	//------------------------------------------------------------------------

	public double normal_cdf (double z) {
		double zz = Math.abs ( z ) ;
		double pdf = Math.exp ( -0.5 * zz * zz ) / Math.sqrt ( 2.0 * 3.141592653589793 ) ;
		double t = 1.0 / (1.0 + zz * 0.2316419) ;
		double poly = ((((1.330274429 * t - 1.821255978) * t + 1.781477937) * t -
							0.356563782) * t + 0.319381530) * t ;
		return (z > 0.0)  ?  1.0 - pdf * poly  :  pdf * poly ;
	}


	/**
	 * This method is called on every bar update and here the indicator value is computed.
	 *
	 * Unlike in MT4 you don't compute indicator values for multiple bars in a loop, 
	 * you need to compute value only for the latest (current) bar.
	 * Trading engine will take care of calling this method for every bar in the chart.
	 *
	 * Actual bar for which the indicator value is computed is stored in CurrentBar variable. 
	 * If 0, it means it is the very first bar of the chart.
	 */
	@Override
	protected void OnBarUpdate() throws TradingException {
    	int barsReq = Math.max(Lookback, ATRLength);
		double indiValue = 0;
		double sum = 0;
		double denom = Indicators.LogATR(Chart, ATRLength).Value.get(1);

		if (CurrentBar < barsReq) {
			// what to set when there are less bars than indicator period parameter
			indiValue = 0;
		} else {
			
			double CurrentClose = Chart.Close.get(0);

			for (int k = 1; k < Lookback+1; k++) {
				sum += Math.log(Chart.Close.get(k));
			}

			sum /= Lookback;

			if (denom > 0) {
				denom *= Math.sqrt(Lookback+1);
				indiValue = (Math.log(CurrentClose) - sum) / denom;
				indiValue = 100 * normal_cdf(1.0 * indiValue) - 50;
			} else {
				indiValue = 0;
			}

		}
		
		// set indicator value to a Value output array.
		// The array is zero based, index 0 means it is value at current bar.
		Value.set(0, indiValue);
	}
}