package SQ.Blocks.Indicators.CSSAMarketRegime;

import com.strategyquant.lib.*;
import com.strategyquant.datalib.*;
import com.strategyquant.tradinglib.*;
import SQ.Calculators.AverageCalculator;
import SQ.Calculators.HighestCalculator;
import SQ.Calculators.LowestCalculator;

import SQ.Internal.IndicatorBlock;

/**
 * Indicator name as it will be displayed in UI, and its return type.
 * Possible return types:
 * ReturnTypes.Price - indicator is drawn on the price chart, like SMA, Bollinger Bands etc.
 * ReturnTypes.Number - indicator is drawn on separate chart, like CCI, RSI, MACD
 * ReturnTypes.PriceRange - indicator is price range, like ATR.
 */
@BuildingBlock(name="(CMR) CSSAMarketRegime", display="CSSAMarketRegime(@Chart@#HLSumPeriod#,#HLPeriod#,#AvgPeriod#,#PercRankPeriod#)[#Shift#]", returnType = ReturnTypes.Number)
@Indicator(oscillator=true, middleValue=50, min=0, max=100, step=0.1)
public class CSSAMarketRegime extends IndicatorBlock {

	@Parameter
	public ChartData Input;


	@Parameter(defaultValue="10", isPeriod=true, minValue=2, maxValue=1000, step=1)
	public int HLSumPeriod;

	@Parameter(defaultValue="10", isPeriod=true, minValue=2, maxValue=1000, step=1)
	public int HLPeriod;

	@Parameter(defaultValue="60", isPeriod=true, minValue=2, maxValue=1000, step=1)
	public int AvgPeriod;

	@Parameter(defaultValue="120", isPeriod=true, minValue=2, maxValue=1000, step=1)
	public int PercRankPeriod;

	@Output
	public DataSeries Value;

	@Buffer
	public DataSeries AvgMetric;

	private HighestCalculator highestCalculator;
	private LowestCalculator lowestCalculator;
	private AverageCalculator averageCalculator;

	//------------------------------------------------------------------------
	//------------------------------------------------------------------------
	//------------------------------------------------------------------------

	/**
	 * This method is called on every bar update and here the indicator value is computed.
	 *
	 * Unlike in MT4 you don't compute indicator values for multiple bars in a loop, 
	 * you need to compute value only for the latest (current) bar.
	 * Trading engine will take care of calling this method for every bar in the chart.
	 *
	 * Actual bar for which the indicator value is computed is stored in CurrentBar variable. 
	 * If 0, it means it is the very first bar of the chart.
	 */
	@Override
	protected void OnInit() throws TradingException {
		highestCalculator = new HighestCalculator(HLPeriod);
		lowestCalculator = new LowestCalculator(HLPeriod);
		averageCalculator = new AverageCalculator(AverageCalculator.SMA,AvgPeriod);
		
	}


	@Override
	protected void OnBarUpdate() throws TradingException {
      double HL=0;
	  double HLsum =0;
	  double TenDayRange = 0;
	  double RawMetric=0;
	  double pRank=0;


		if (CurrentBar < Math.max(AvgPeriod,Math.max(PercRankPeriod,Math.max(HLPeriod,HLSumPeriod)))) {
			// what to set when there are less bars than indicator period parameter
			Value.set(0, 0);
			AvgMetric.set(0, 0);

		} else {

			/// step 1
			HL = Input.High.get(0) - Input.Low.get(0);
			/// step 2
			for(int i = 0;i<=HLSumPeriod;i++){
				
				HLsum += (Input.High.get(i) - Input.Low.get(i));
			}
			/// step 3

			highestCalculator.onBarUpdate(Input.High.get(0), CurrentBar);
			lowestCalculator.onBarUpdate(Input.Low.get(0), CurrentBar);
			TenDayRange = highestCalculator.getHighestValue()-lowestCalculator.getLowestValue();
			
			///step 4

			RawMetric = HLsum/TenDayRange;

			/// step 5

			averageCalculator.onBarUpdate(RawMetric, CurrentBar);
			AvgMetric.set(0,averageCalculator.getValue()); // adding to buffer
			
			/// step 6
			double Rank = 0;
			for(int p = 1;p<=PercRankPeriod;p++){
				
				if(AvgMetric.get(0)>AvgMetric.get(p)) Rank++;
			}
			pRank = (double)Rank/PercRankPeriod*100;

			
			Value.set(0, pRank);
		}

	}
}